extern crate git2;

use std::env;
use std::fs::File;
use std::io::Write;
use std::path::Path;

fn main() {
    let mut dirty_repo = String::new();

    match git2::Repository::discover("./") {
        Ok(repo) => {
            let remotes = match repo.remotes() {
                Ok(array) => array,
                Err(e) => panic!("No remotes found. Details: {}", e)
            };

            let head = match repo.head() {
                Ok(reference) => match reference.peel_to_tree() {
                    Ok(tree) => tree,
                    Err(e) => panic!("Could not peel head to tree! Details: {}", e)
                }
                Err(e) => panic!("Could not find head. Details: {}", e)
            };

            let mut diff_option = git2::DiffOptions::new();
            diff_option.include_untracked(true);

            let _diff = match repo.diff_tree_to_workdir(Some(&head), Some(&mut diff_option)) {
                Ok(result) => {
                    diff_check();
                    dirty_repo = head.id().to_string() + " (dirty)";
                    result
                },
                Err(e) => panic!("Could not create diff. Details: {}", e)
            };
            generate_build_info(dirty_repo, remotes);
        },
        Err(e) => println!("No git repo found. Details: {}", e)
    }
}

fn diff_check() {
    println!("cargo:warning=ambassade-debug: Current repository is dirty, meaning that debug info from this build will be less reliable.");
}

fn generate_build_info(sha: String, _remotes: git2::string_array::StringArray) {
    let out_dir = env::var("OUT_DIR").unwrap();
    let dest_path = Path::new(&out_dir).join("build_info.rs");

    let mut content = String::from("/// Generated by a Cargo build script. Indicates the git status of the working directory.\n");
    content.push_str("pub fn sha() -> &'static str {");
    content.push_str("\"");
    content.push_str(sha.as_str());
    content.push_str("\"\n}");

    let mut file = File::create(&dest_path).unwrap();

    file.write_all(content.as_bytes()).unwrap();
}
